/*
Assembled by Andrew Roth, Future Cinema Lab at York University
This project is released under GPL license.

FLARManager by Eric Socolofsky: http://words.transmote.com/wp/flarmanager/ 
FLARToolkit courtesy of Squoosha and Nyatla: http://www.libspark.org/wiki/saqoosha/FLARToolKit/en

FLARToolKit is free to use for non-commercial applications under the GPL license. 
This means the complete source code for your application must be made available to anyone 
that asks for it. FLARToolKit is based on the ARToolKit library under the GPL license and
so the source code for any FLARToolKit applications that are made needs to be GPL as well.

Based on Tutorials from Lee Brimelow: http://theflashblog.com/
Alpha transparency and button tutorial from Andrew Garrahan: http://www.garrahan.org/papervideo-part3-alpha.php
*/

package
{
	//import packages for FLARManager
	
	import com.transmote.flar.FLARManager;
	import com.transmote.flar.marker.FLARMarker;
	import com.transmote.flar.marker.FLARMarkerEvent;
	import com.transmote.flar.utils.geom.FLARPVGeomUtils;
	
	//Import standard Flash Libraries
	
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.net.URLLoader;
	import flash.net.URLRequest;
	import flash.text.TextField;
	
	//Import your libsparks libraries
	
	import org.libspark.flartoolkit.support.pv3d.FLARCamera3D;
	import org.libspark.flartoolkit.utils.Global;
	
	//Import your Papervision3D libraries
	import org.papervision3d.events.FileLoadEvent;
	import org.papervision3d.lights.PointLight3D;
	import org.papervision3d.materials.BitmapFileMaterial;
	import org.papervision3d.materials.MovieMaterial;
	import org.papervision3d.materials.utils.MaterialsList;
	import org.papervision3d.objects.DisplayObject3D;
	import org.papervision3d.objects.parsers.KMZ;
	import org.papervision3d.objects.primitives.Cube;
	import org.papervision3d.objects.primitives.Plane;
	import org.papervision3d.objects.primitives.Sphere;
	import org.papervision3d.render.LazyRenderEngine;
	import org.papervision3d.scenes.Scene3D;
	import org.papervision3d.view.Viewport3D;
	
	//metadata for stage setup
	[SWF(width='640', height='480', backgroundColor='#000000', frameRate='40')]
	
	public class multivideo extends Sprite
	{
		//declare your variables here.
		private var fm:FLARManager;
		private var scene:Scene3D;
		private var view:Viewport3D;
		private var camera:FLARCamera3D;
		private var lre:LazyRenderEngine;
	
		private var con:DisplayObject3D;
		private var marker:FLARMarker;
		private var v:Vid;
		private var p1:Plane;
		private var p2:Plane;
		private var p3:Plane;
		private var p4:Plane;
		private var c:Cube;
		private var s:Sphere;
		private var loadBall:Sprite;
		private var img1:String;
		private var img2:String;
		private var cuMat:MaterialsList;
			
		private var myMdl:KMZ;
		private var mdl:String;
		private var light:PointLight3D;

		private var vlist:XML;
		private var myXMLLoader:URLLoader;
		private var my_urls:String;
		private var rnd1:Number;
		private var rnd2:Number;
		private var video1:Sprite;
		private var video2:Sprite;
		private var bg:Sprite;
		private var bgFr:Sprite;
		private var label:TextField;
		private var labelFr:TextField;
		private var isLoaded:Number;
		
		public function multivideo()
		{
			
			//begin by parsing the playlist.xml file, when the file is complete perform function processXML
			var myXMLLoader:URLLoader = new URLLoader();
			myXMLLoader.load(new URLRequest("playlist.xml"));
			myXMLLoader.addEventListener(Event.COMPLETE, processXML);

		}
		
		private function initFLAR():void
		{
			//setup FLARManager, parse flarConfig.xml and add the FLARM events Add/remove
			fm = new FLARManager("data/flarConfig.xml");
			fm.addEventListener(FLARMarkerEvent.MARKER_ADDED, onAdded);
			fm.addEventListener(FLARMarkerEvent.MARKER_REMOVED, onRemoved);
			fm.addEventListener(Event.INIT, init2D1);
			fm.addEventListener(Event.INIT, init3D);
			
			//this will help us determine if there is any marker in the scene when it's reloaded.
			Global.mkr = -1;
			Global.currentSet= 0;
			
			//child FLARManager source to sprite
			addChild(Sprite(fm.flarSource));
		}

		
		private function onAdded(e:FLARMarkerEvent):void
		{	
			//if marker in FLARConfig is recognized set the global marker value and make elements visible, start video
			marker = e.marker;
			Global.mkr = e.marker.patternId;
			p1.visible = true;
			p2.visible = true;
			p3.visible = true;
			p4.visible = true;
			s.visible = true;
			myMdl.visible = true;
			v.vid.play();
			
		}
		
		private function onRemoved(e:FLARMarkerEvent):void
		{
			//if marker is removed from scene reset marker and make global marker -1.  Hide elements, stop video.
			marker = null;
			Global.mkr = -1;
			p1.visible = false;
			p2.visible = false;
			p3.visible = false;
			p4.visible = false;
			s.visible = false;
			myMdl.visible = false;
			v.vid.stop();
			
		}
		
		private function processXML(e:Event): void {
			//this is where the XML playlist is parsed
			var vlist:XML = new XML(e.target.data);
			var vlistScaleAttribute:XML = new XML(e.target.data);
			
			XML.ignoreWhitespace = true;
			
			//grab XML attribte 'scale' from 'model' and add to array for later use
			var scaleAttributes:XMLList = vlistScaleAttribute.model.attributes();
			var i1s:Number = 0;
			for each (var modelScale:XML in scaleAttributes) {
				if (modelScale.name() == "scale") {
					Global.attSca[i1s]= modelScale;
					i1s++;
				}
			}
			
			//grab XML attribte 'pos' from 'model' and add to array for later use
			var i1p:Number = 0;
			for each (var modelPos:XML in scaleAttributes) {
				if (modelPos.name() == "pos") {
					Global.attPos[i1p]= modelPos;
					i1p++;
				}
			}
			
			//populate playlist arrays
			for (var i1:int=0;i1<vlist.video.length();i1++) 
			{
				Global.myVidpl[i1]= vlist.video[i1];
			}
			
			for (var i2:int=0;i2<vlist.model.length();i2++) 
			{
				Global.myMdlpl[i2]= vlist.model[i2];
			}
			for (var i3:int=0;i3<vlist.image.length();i3++) 
			{
				Global.myImgpl[i3]= vlist.image[i3];
			}
			
			//find lowest length of the 3 arrays to avoid errors if there are not Video, Model and image components in scene
			var tempArray:Array = new Array();
			var sortedArray:Array = new Array();
			var tempNum:Number = new Number();
			tempArray[0] = vlist.model.length();
			tempArray[1] = vlist.video.length();
			tempArray[2] = vlist.image.length();
			
			sortedArray = tempArray.sort(Array.NUMERIC);
			Global.lgth = sortedArray[0];
			
			trace(Global.lgth);
			
			//once arrays are populated begin initializing FLAR
			initFLAR();
		}
		
		private function init3D(e:Event):void
		{	
			//container will be null if loaded for the first time, remove elements only if they are present.
			isLoaded = 0;
			if((con != null)&&(con.toString != null))
			{    
			//loadBall.parent.removeChild(loadBall);
     		 p1.parent.removeChild(p1);
     		 p2.parent.removeChild(p2); 
     		 p3.parent.removeChild(p3);
     		 p4.parent.removeChild(p4);
     		 s.parent.removeChild(s);
     		 myMdl.parent.removeChild(myMdl); 
     		 v.vid.stop();  
     		 view.parent.removeChild(view);
     		 //Scroll though each series in set
     		 Global.currentSet += 1;
     		 //loop back to the beginning of the set without exceeding total # of videos
     		 Global.currentSet = Global.currentSet % Global.lgth;
 			 }
 			 //implement this if generating random scene
			//var currentSet:Number = Math.floor(Math.random()*Global.lgth);
			
			//uncomment this for testing a particular scene
			//Global.currentSet = 5;
			
			//setup 3D view and FLARcamera scene
			scene = new Scene3D();
			camera = new FLARCamera3D(fm.cameraParams);
			camera.z = -30;
			view = new Viewport3D(640, 480, true);
			lre = new LazyRenderEngine(scene, camera, view);
			con = new DisplayObject3D();
			
			//dim the main display during loading
			this.alpha = 0.5;
		
			//here's where you load your video based on which scene is showing
		
			v = new Vid();
			var vidSourceTmp:String = new String();
			vidSourceTmp = "media/"+Global.myVidpl[Global.currentSet];
			v.vid.source = vidSourceTmp;
			v.vid.stop();
			
			//load the placeholders for current scene	
			
			var img1:String = new String();
			var img2:String = new String();
			
			img1 = "media/"+Global.myImgpl[Global.currentSet];
			img2 = "media/"+Global.myImgpl[Global.currentSet];
			mdl = "model/"+Global.myMdlpl[Global.currentSet];
			
			myMdl = new KMZ();
			//load kmz model and listen for LOAD_COMPLETE **LOAD_PROGRESS not working for kmz.
			myMdl.addEventListener(FileLoadEvent.LOAD_COMPLETE, loadProgressHandler);
			myMdl.load(mdl);
			myMdl.scale = Global.attSca[Global.currentSet];
			myMdl.rotationZ = -90;
			myMdl.rotationY = -120;
			myMdl.visible = false;
			trace(Global.attPos[Global.currentSet]);
			myMdl.x = Global.attPos[Global.currentSet]; //+up -down
			myMdl.y = 60; //-left +right
			myMdl.z = 80; //+front to -back
			
			//setup sky with alpha transparency
			var mat4:BitmapFileMaterial = new BitmapFileMaterial("media/cloudsSM.png", true);
			s = new Sphere(mat4, 30, 30, 30);
			mat4.doubleSided = false;
			s.rotationZ = -90;
			s.scaleX = 5.0;
			s.scaleY = 3.6;
			s.scaleZ = 5.0;
			//p1.x = 100; //+ to position behind plane on bottom
			//s.x = -60; //(towards floor)
			//s.y = -80; //left right
			//s.useOwnContainer = true;
			s.x = 30;
			s.alpha = 0.4;
			s.z = -100;
			s.visible = false;
			
			//setup movie material for alpha transparency set all arguments to true.
			var mat1:MovieMaterial = new MovieMaterial(v, true, true);
			p1 = new Plane(mat1, 320, 240, 2, 2);
			mat1.doubleSided = true;
			p1.scaleY = -0.3;
			p1.scaleX = 0.3;
			p1.rotationZ = 90;
			//p1.x = 100; //+ to position behind plane on bottom
			p1.y = -60;
			p1.z = 80;
			p1.visible = false;

			//setup back picture with alpha transparency
			var mat2:BitmapFileMaterial = new BitmapFileMaterial(img2, true);
			//var mat2:BitmapFileMaterial = new BitmapFileMaterial("media/torontoskyline.png", false);
			p2 = new Plane(mat2, 320, 240, 2, 2);
			p2.scaleY = -0.53;
			p2.scaleX = 0.7;
			p2.rotationZ = 90; //was -90
			//p2.rotationY = -180;
			//p2.x = ; //was -40
			p2.visible = false;
			
				//setup back picture with alpha transparency
			var mat5:BitmapFileMaterial = new BitmapFileMaterial("media/tmp.jpg", true);
			//var mat2:BitmapFileMaterial = new BitmapFileMaterial("media/torontoskyline.png", false);
			p4 = new Plane(mat5, 320, 240, 2, 2);
			p4.scaleY = -0.2;
			p4.scaleX = 0.5;
			p4.rotationZ = 90; //was -90
			//p2.rotationY = -180;
			p4.x = 95; //was -40
			p4.visible = false;
	
			//setup bottom picture with alpha transparency
			var mat3:BitmapFileMaterial = new BitmapFileMaterial("media/Toronto_Landsat.png", true);
			p3 = new Plane(mat3, 320, 240, 2, 2);
			mat3.doubleSided = true;
			p3.scaleY = -0.8;
			p3.scaleX = -0.7;
			p3.rotationX = 90;
			p3.rotationZ = -90;
			p3.alpha = 0.2;
			//p1.x = 100; //+ to position behind plane on bottom
			p3.x = -60; //(towards floor)
			p3.y = 0; //left right
			p3.z = 90;
			p3.visible = false;

			/*  lighting point probably not necessary for our textured images but here's where you'd implement it
			var light:PointLight3D = new PointLight3D();
			light.x = 1000; //was 1000
			light.y = 1000;
			light.z = -1000;
  			*/
  			
  			//setup display containter 
  			//Interesting trick
			//con.useOwnContainer = true;
			//con.alpha = 0.2;
			//OR
			//view.alpha = 0.2; makes everything ghostly
			
			con.addChild(myMdl);
			con.addChild(s);
			con.addChild(p1);
			con.addChild(p2);
			con.addChild(p3);
			con.addChild(p4);
			
			scene.addChild(con);
			
			addChild(view);
			
			//setup listener for render event
			addEventListener(Event.ENTER_FRAME, loop);
		}
		
		//this is the completed event, reload all elements to marker to prempt addmarker, undim background
		private function loadProgressHandler(e:Event):void
		{
			//trace(e);
			if (Global.mkr == 0){
			p1.visible = true;
			p2.visible = true;
			p3.visible = true;
			p4.visible = true;
			s.visible = true;
			myMdl.visible = true;
			v.vid.play();
			}
			this.alpha = 1.0;
			
		}
		
		private function init2D1(e:Event):void
		{	
			//let's make a simple button to change the scene just as in Andrew Garrahan's example but in the top right
			var video1:Sprite = makeButton('Changez la Scène');
			video1.addEventListener(MouseEvent.CLICK, init3D);
			video1.buttonMode = true;
			
			addChild(video1);
		

			function makeButton(title:String) : Sprite 
			{
				
				var label:TextField = new TextField();
				label.textColor = 0xFFFFFF;
				label.text = title;
				label.height = 30;
				label.x = 645-label.width;
				label.selectable = false;
				
				var bg:Sprite = new Sprite();
				bg.graphics.beginFill(0x000000);
				bg.graphics.drawRect(640-label.width,0,label.width,20);
				bg.alpha = 0.8;
				bg.addEventListener(MouseEvent.MOUSE_OVER, onState );
				bg.addEventListener(MouseEvent.MOUSE_OUT, offState );
				
					bg.addChild(label);
				function onState(e:Event) : void {		
					label.textColor = 0x000000;
					bg.graphics.clear();
					bg.graphics.beginFill(0xFFFFFF);
					bg.graphics.drawRect(640-label.width,0,label.width,20);
				}
				function offState(e:Event) : void {		
					label.textColor = 0xFFFFFF;
					bg.graphics.clear();
					bg.graphics.beginFill(0x000000);
					bg.graphics.drawRect(640-label.width,0,label.width,20);
				}
				
				return bg;
				
			}		
		}
		
		//render loop
		private function loop(e:Event):void
		{
			if(marker != null)
			{ 
			s.rotationY -= 0.6;
			con.transform = FLARPVGeomUtils.convertFLARMatrixToPVMatrix(marker.transformMatrix);
			
			}
			lre.render();
		}
	}
}







